#define NOINCLn
/*******************************************************************
 * Copyright (C) 2000 SSI. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
/**************************************************
 * RCSId: $Id$
 *
 * Calculate current observation point for polar satellites 
 * Project: SSI
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/**************************************************
  Main functions:
    xy2lonlat(): translates (x,y) of picture into (lon,lat)
    calc_orbitconst()
 **************************************************/
/**************************************************
  Used formulas to calculate the satellite orbit:

    Used symbols:
      h=height sat
      Rearth=radius earth
      g0=gravity
      to=time 1 loop (in sec)
      lpd=loops per day = 24*3600/to
      orbit_nr=# orbits since reference time (=time Kepler data)
      dt=time elapsed since reference time
      orbit_angle=remainder orbit (orbit_nr - n*2pi)
      k2_n=deviation orbit north-south
      k2_w=deviation orbit west-east

    Common physics:
      to=2*pi*sqrt(Rearth/g0) * (1+h/Rearth)**(1.5)

          { [         to           ](2/3)     }
      h = { [ -------------------- ]      - 1 } * Rearth
          { [ 2*pi*sqrt(Rearth/g0) ]          }


    Satellite movement:
                                      
                     [  Rearth  ](3.5)            2
      k2_n = Const * [ -------- ]      * {2.5*(cos(incl)-1) + 2}
                     [ h+Rearth ]

      dh_n=dt*k2_n

                     [  Rearth  ](3.5)            
      k2_w = Const * [ -------- ]      * cos(incl)
                     [ h+Rearth ]

      dh_w=dt*k2_w

      orbit_nr=(lpd + decay*dt)*dt

      orbit_angle=(orbit_nr-int(orbit_nr))*2*pi + perigee + anomaly

      x1=sin(orbit_angle+dh_n*dt)*cos(incl)
      y1=cos(orbit_angle+dh_n*dt)
      z1=sin(orbit_angle+dh_n*dt)*sin(incl)

  Take into account sensor movement.
  beta=angle "sat -> earth-middle -> current observation point"
      x=x1*cos(beta)+sin(inclination)*sin(beta);
      y=y1*cos(beta);
      z=z1*cos(beta)-cos(inclination)*sin(beta);

   Translate to lon/lat
      sat_lon=atan(y/x)+raan-dh_w
      sat_lat=asin(z)

    Earth rotation:
      rem_e=(rot_year-int(rot_year))*2*pi
      earth_lon=rem_e + Earth_offsetangle
      earth_lat=0;

    Relative rotation. If beta=0 then this is the sub-satellite point.
      rel_lon=sat_lon-earth_lon
      rel_lat=sat_lat

 **************************************************/
#include <math.h>
#include "xtrack.h"
#include "orbit.h"

/*************************************
 * Pre-calculate some orbit constants
 *************************************/
int calc_orbitconst(KEPLER *kepler,ORBIT *orbit)
{
#ifdef NOINCL
kepler->inclination=PI/2.;
#endif
  if (kepler->motion<0.1) return 0;

  orbit->loop_time=24.*3600./kepler->motion;
  orbit->height=(pow(orbit->loop_time/PIx2/sqrt(Rearth/G0),2./3.)-1)*Rearth;

  orbit->k2_n=Const * pow(Rearth/(Rearth+orbit->height),3.5) * 
                          (2.5*pow(cos(kepler->inclination),2.)-0.5);

  orbit->k2_w=Const * pow((Rearth/(Rearth+orbit->height)),3.5) * 
                                            cos(kepler->inclination);

  orbit->k2_n=D2R(orbit->k2_n);
  orbit->k2_w=D2R(orbit->k2_w);

/* Translate reference time into tm struct */
  orbit->ref_tm.tm_isdst=0;
  orbit->ref_tm.tm_year=kepler->epoch_year;
  orbit->ref_tm.tm_yday=kepler->epoch_day-1;
  yday2mday_mon(&orbit->ref_tm);
  orbit->ref_tm.tm_hour=  (kepler->epoch_day-1-orbit->ref_tm.tm_yday)*24;
  orbit->ref_tm.tm_min = ((kepler->epoch_day-1-orbit->ref_tm.tm_yday)*24-
                                               orbit->ref_tm.tm_hour)*60;
  orbit->ref_tm.tm_sec =(((kepler->epoch_day-1-orbit->ref_tm.tm_yday)*24-
                                               orbit->ref_tm.tm_hour)*60-
                                               orbit->ref_tm.tm_min )*60;

  orbit->ref_time=mktime_ntz(&orbit->ref_tm);
  orbit->ref_tm.tm_wday=0;
  orbit->ref_ms=0;

/* Set valid-flag */
  orbit->valid=TRUE;
  return 1;
}

/*************************************
 * Calculate position of satellite from 
 *   current time and reference time (ref-time is in KEPLER-data)
 * Result in pos_sat->x,y,z
 * raan and dh_w NOT taken into account. These have to be added to the lon.
 * Return: 0 if calculation has failed 
 *         1 if OK
 *************************************/
int calcpossat(struct tm *cur_tm, int cur_ms,  /* current time */
               KEPLER *kepler,                 /* kepler data */
               ORBIT *orbit,                   /* orbit data */
               POINT *pos_sat)                 /* position satellite */
{
  double dift;
  double loop_nr;
  float orbit_angle,dh_n,dh_w;

/* Determine time difference with reference in sec */
  dift=mktime_ntz(cur_tm)-orbit->ref_time;
  dift=dift+(float)(cur_ms-orbit->ref_ms)/1000.;

/* Translate difference into days */
  dift/=(24.*3600.);

/* Calculate # loops since ref-time. */
  loop_nr=(kepler->motion+kepler->decay_rate*dift)*dift; 
  if (ABS(loop_nr/10) >= FLT_MAX) return 0;     /* loop_nr too large -> error */

/* Calc orbit-angle in radians */
  orbit_angle=(loop_nr-(int)loop_nr)*PIx2;

/* Add starting point */
  orbit_angle=orbit_angle+kepler->perigee+kepler->anomaly;

/* Shift result to [0 ... pi*2] */
  while (orbit_angle<0)    orbit_angle+=PIx2;
  while (orbit_angle>PIx2) orbit_angle-=PIx2;

/* Determine corrections */
  dh_n=dift*orbit->k2_n;
  dh_w=dift*orbit->k2_w;

  dh_n=dh_n-(int)(dh_n/PIx2)*PIx2;
  dh_w=dh_w-(int)(dh_w/PIx2)*PIx2;
  orbit->dh_w=dh_w;

/* Translate to orthonormal coordinates */
  pos_sat->x=sin(orbit_angle+dh_n)*cos(kepler->inclination);
  pos_sat->y=cos(orbit_angle+dh_n);
  pos_sat->z=sin(orbit_angle+dh_n)*sin(kepler->inclination);

#define BEREKEN_MOM_HOOGTEn
#ifdef BEREKEN_MOM_HOOGTE
{
float loop_time=24.*3600./(kepler->motion+kepler->decay_rate*dift);
float height=(pow(loop_time/PIx2/sqrt(Rearth/G0),2./3.)-1)*Rearth;
printf("%f  %f  %f\n",height,kepler->decay_rate,(float)dift);
}
#endif
  return 1;
}


/*************************************
 * Calculate observation point of satellite.
 * Takes into account sensor movement.
 * Result in pos_sat->lon,lat
 *************************************/
void calc_satobs(KEPLER *kepler,ORBIT *orbit,
                 POINT *pos_sati,POINT *pos_sato,float beta)
{
/* No need to calc. if beta=0. Just to speed up calcs. */

  if (beta)
  {
    pos_sato->x=pos_sati->x*cos(beta)+sin(kepler->inclination)*sin(beta);
    pos_sato->y=pos_sati->y*cos(beta);
    pos_sato->z=pos_sati->z*cos(beta)-cos(kepler->inclination)*sin(beta);
  }
  else
  {
    pos_sato->x=pos_sati->x;
    pos_sato->y=pos_sati->y;
    pos_sato->z=pos_sati->z;
  }
}

float x2earthangle(float ,SCANDIR , int ,int,float );
void calc_x2satobs(KEPLER *kepler,ORBIT *orbit,
                   POINT *pos_sati,POINT *pos_sato,
                   int width_original,float max_sens_angle,
                   int x)
{
  float beta;
  beta=x2earthangle(orbit->height,E_W,
                    x,width_original/2,max_sens_angle);
  calc_satobs(kepler,orbit,pos_sati,pos_sato,beta);
}

/*************************************
 * Calculate position of earth since jan 1 
 *************************************/
void calcposearth(struct tm *cur_tm, int ms, POINT *pos_earth)
{
  float days,rot_earth;
/* days since jan 1 */
  days=cur_tm->tm_yday+1+cur_tm->tm_hour/24.+
                             cur_tm->tm_min/1440.+
                             cur_tm->tm_sec/86400.+
                             (float)ms/86400000.;

/* earth rotations since jan 1. Translate to 365.24 days/year. */
  rot_earth=days*LEN_LEAPYEAR/LEN_YEAR;

/* Translate into rotations. Ignore full rotations and add offset. */
  pos_earth->lon=PIx2*(rot_earth-(int)rot_earth)+D2R(Earth_offsetangle);

/* Translate to range [0 ... 2*pi] */
  while (pos_earth->lon>=PIx2) pos_earth->lon-=PIx2;

/* earth rotates in lon direction only */
  pos_earth->lat=0;
}

/*************************************
 * Calculate position of satellite relative to earth
 *************************************/
void calcposrel(KEPLER *kepler,ORBIT *orbit,
                POINT *pos_sat,POINT *pos_earth,POINT *pos_rel)
{
/* Translate to long/lat */

  pos_rel->lon=atan2(pos_sat->x,pos_sat->y)+
                          kepler->raan-orbit->dh_w-pos_earth->lon;
  pos_rel->lat=asin(pos_sat->z);

/* Shift to range [-pi..+pi] */  
  while (pos_rel->lon < -1*PI) pos_rel->lon+=PIx2;
  while (pos_rel->lon >    PI) pos_rel->lon-=PIx2;
  while (pos_rel->lat < -1*PI) pos_rel->lat+=PIx2;
  while (pos_rel->lat >    PI) pos_rel->lat-=PIx2;
}

